#!/bin/bash
# Creation of the avision.desc SANE-description file from the avisoin.c.
# The C-Language sourcecode of the SANE/Avision backend.
#
# (C) Copyright 2002 Ren Rebe - License: GPL

case "$1" in
    -html) mode=html ;;
    -desc) mode=desc ;;
    *)  echo "Usage: -desc -html" ; exit -1 ;;
esac

output_head()
{
    if [ "$mode" = "desc" ] ; then
    cat <<- EOT
; SANE Backend specification file
;
; THIS FILE IS GENERATED FROM THE "avision.c" FILE AUTOMATICALLY - DO NOT EDIT
; THIS FILE MANUALLY !!!
;
; It's basically emacs-lisp --- so ";" indicates comment to end of line.
; All syntactic elements are keyword tokens, followed by a string or
;  keyword argument, as specified.
;
; ":backend" *must* be specified.
; All other information is optional (but what good is the file without it?).

; name of backend
:backend "avision"

; version of backend
EOT

echo ":version \"$1\""

    cat <<- EOT
;status: alpha, beta, stable, new
;:status :stable

; backend's web page
:url "http://www.exactcode.de/oss/avision/index.html"

; name of manpage (if it exists)
:manpage "sane-avision"

:devicetype :scanner
EOT
    else
	echo "<H4>Supported devices by SANE/Avision $1:</H4>"
    fi
}

output_tail()
{
    [ $mode = html ] && echo "</ul>"
}


last_mfg=""

# SANE syntax output generation function
output_device()
{
    mfg="`echo "$@" | cut -d '\"' -f 6`"
    if [ "$first_mfg" ] ; then
	first_mfg=0;
    else
	first_mfg=1;
    fi
    model="`echo "$@" | cut -d '"' -f 8`"

    usbid="`echo "$@" | cut -d '"' -f 5 | \
            sed -e 's/ //g' -e 's/^.//' -e 's/.$//'`"
    
    case "$usbid" in
	"0,0") interface="SCSI" ;;
	*) interface="USB" ;;
    esac
    
    if [[ "$@" == *comment* ]] ; then
	comment="`echo "$@" | sed 's/.*comment="\([^"]*\).*/\1/'`"
    else
	comment=""
    fi
    status="`echo "$@" | sed 's/.*status="\([^"]*\).*/\1/'`"
    
    if [ "$last_mfg" != "$mfg" ] ; then
	new_mfg=1
	last_mfg="$mfg"
	case "$mfg" in
	    "Avision") mfg_url="http://www.avision.com" ;;
	    "Hewlett-Packard") mfg_url="http://www.hp.com" ;;
	    "Minolta") mfg_url="http://www.minolta.com" ;;
	    "Fujitsu") mfg_url="http://www.fujitsu.com" ;;
	    "Kodak") mfg_url="http://www.kodak.com" ;;
	    *) mfg_url="" ;;
	esac
    else
	new_mfg=0
    fi

    if [ "$mode" = "desc" ] ; then
	if [ $new_mfg = 1 ] ; then
	    echo
	    echo ":mfg \"$mfg\""
	    [ "$mfg_url" ] && echo ":url \"$mfg_url\""
	    echo
	fi
        echo ":model \"$model\""
	echo ":interface \"$interface\""
	[[ "$comment" ]] &&  echo ":comment \"$comment\""
	echo ":status :$status"
	echo
    else
	if [ $new_mfg = 1 ] ; then
	    [[ "$first_mfg" = 0 ]] && echo "</ul>"
	    echo
	    if [ "$mfg_url" ] ; then
	      echo "<B><a href=\"$mfg_url\">$mfg</a>:</B>"
	    else
	      echo "<B>$mfg:</B>"
	    fi
	    echo "<ul>"
	fi
	echo -n "<li>$model ($interface, $status"
	[ "$usbid" != "0,0" ] && echo -n ", USB ID: $usbid"
	[ "$comment" ] && echo -n ", $comment"
	echo ")</li>"
    fi
}

( # only process after a start tag - and store version
start_pattern="*Avision_HWEntry*"
start=0
while read in ; do
	# extract versin information
	if [[ "$in" = *BUILD* ]] ; then
	    new_ver="`echo "$in" | sed 's/.*BACKEND_BUILD \([0-9]*\) .*/\1/'`"
	    if [ "$ver" = "" ] ; then
	    	ver=$new_ver
		echo "VERSION: $ver"
	    fi
	fi
	[[ $in == $start_pattern  ]] && start=1
	[ $start = 1 ] && echo "$in"
done
) | ( # only process up to end tag
end_pattern="*NULL, NULL*"
end=0
while read in ; do
	[[ $in == $end_pattern  ]] && end=1
	[ $end = 0 ] && echo "$in"
done
) | (
# state-machine to trace open and closed curlies to output the scanner-lines
open=0
one_found=0
header_written=0
while read line ; do
    if [[ $line == VERSION* ]] ; then
	output_head "Build: ${line//VERSION: /}"
    fi
    if [[ $line == *{* ]] ; then
	[ $open = 0 ] && [ $one_found = 1 ] && output_device "$text"
	one_found=1
	open=1
	text=""
    fi
    text="$text $line"
    if [[ $line == *}* ]] ; then
	open=0
    fi
done
    [ $one_found = 1 ] && output_device "$text"
    output_tail
)
